@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2017 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: ArgV.cc,v 1.22 2010/12/28 11:46:25 lav Exp $ */
-
 #include <config.h>
 
 #include "misc.h"
@@ -58,18 +56,37 @@ void ArgV::back()
       ind--;
 }
 
-char *ArgV::Combine(int start,int end) const
+xstring& ArgV::CombineTo(xstring& res,int start,int end) const
 {
-   xstring res("");
+   res.nset("",0);
    if(!end)
       end=Count();
    if(start>=end)
-      return res.borrow();
+      return res;
    for(;;)
    {
       res.append(getarg(start++));
       if(start>=end)
-	 return(res.borrow());
+	 return(res);
+      res.append(' ');
+   }
+}
+
+xstring& ArgV::CombineShellQuotedTo(xstring& res,int start) const
+{
+   res.nset("",0);
+   if(start>=Count())
+      return res;
+   for(;;)
+   {
+      for(const char *arg=String(start++); *arg; arg++)
+      {
+	 if (is_shell_special(*arg))
+	    res.append('\\');
+	 res.append(*arg);
+      }
+      if(start>=Count())
+	 return(res);
       res.append(' ');
    }
 }
