@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: DummyProto.h,v 1.12 2009/03/20 12:28:04 lav Exp $ */
-
 #ifndef DUMMYPROTO_H
 #define DUMMYPROTO_H
 
@@ -31,10 +29,11 @@ public:
    int Done();
    const char *GetProto() const;
    FileAccess *Clone() const;
-   int Read(void *buf,int size);
+   int Read(Buffer *buf,int size);
    int Write(const void *buf,int size);
    int StoreStatus();
    void Reconfig(const char *) {}
+   void Login(const char *,const char *) {}
 
    ListInfo *MakeListInfo(const char *path);
    DirList *MakeDirList(ArgV *);
