@@ -344,6 +344,7 @@ void SFtp::Init()
    size_write=0x8000;
    use_full_path=false;
    flush_timer.Set(0,500);
+   max_out_of_order=64;
 }
 
 SFtp::SFtp() : SSH_Access("SFTP:")
@@ -824,7 +825,7 @@ void SFtp::HandleExpect(Expect *e)
 	    SetError(NO_FILE,strerror(ENOTDIR));
 	    break;
 	 }
-	 if(mode==CHANGE_DIR && GetExpectCount(Expect::CWD)==0)
+	 if(mode==CHANGE_DIR && !HasExpect(Expect::CWD))
 	 {
 	    cwd.Set(file);
 	    eof=true;
@@ -876,7 +877,7 @@ void SFtp::HandleExpect(Expect *e)
 	    d->GetData(&b,&s);
 	    LogNote(9,"data packet: pos=%lld, size=%d",(long long)r->pos,s);
 	    file_buf->Put(b,s);
-	    if(d->Eof())
+	    if(d->Eof() || eof)
 	       goto eof;
 	    if(r->len > unsigned(s))   // received less than requested?
 	    {
@@ -892,7 +893,7 @@ void SFtp::HandleExpect(Expect *e)
 	 {
 	    LogNote(9,"put a packet with id=%d on out-of-order chain (need_pos=%lld packet_pos=%lld)",
 	       reply->GetID(),(long long)(pos+file_buf->Size()),(long long)r->pos);
-	    if(ooo_chain.count()>=64)
+	    if(ooo_chain.count()>=max_out_of_order)
 	    {
 	       LogError(0,"Too many out-of-order packets");
 	       Disconnect();
@@ -938,7 +939,7 @@ void SFtp::HandleExpect(Expect *e)
 	       file_set->Add(info);
 	    }
 	 }
-	 if(r->Eof())
+	 if(r->Eof() || eof)
 	    goto eof;
       }
       else
@@ -948,12 +949,13 @@ void SFtp::HandleExpect(Expect *e)
 	    if(((Reply_STATUS*)reply)->GetCode()==SSH_FX_EOF)
 	    {
 	    eof:
-	       if(!eof)
-		  LogNote(9,"eof");
 	       eof=true;
 	       state=DONE;
-	       if(file_buf && ooo_chain.count()==0)
+	       if(file_buf && ooo_chain.count()==0 && !HasExpectBefore(reply->GetID(),Expect::DATA))
+	       {
+		  LogNote(9,"eof");
 		  file_buf->PutEOF();
+	       }
 	       break;
 	    }
 	 }
@@ -1062,8 +1064,11 @@ int SFtp::HandleReplies()
       }
    }
 
-   if(ooo_chain.count()==0 && eof && file_buf && !file_buf->Eof())
+   if(eof && file_buf && !file_buf->Eof() && ooo_chain.count()==0 && !HasExpect(Expect::DATA))
+   {
+      LogNote(9,"eof");
       file_buf->PutEOF();
+   }
 
    if(recv_buf->Size()<4)
    {
@@ -1118,6 +1123,7 @@ SFtp::Expect *SFtp::FindExpectExclusive(Packet *p)
       e->reply=p;
    return e;
 }
+
 void SFtp::CloseExpectQueue()
 {
    for(Expect *e=expect_queue.each_begin(); e; e=expect_queue.each_next())
@@ -1144,12 +1150,26 @@ void SFtp::CloseExpectQueue()
    }
 }
 
-int SFtp::GetExpectCount(Expect::expect_t tag)
+bool SFtp::HasExpect(Expect::expect_t tag)
 {
-   int count=0;
    for(Expect *e=expect_queue.each_begin(); e; e=expect_queue.each_next())
-      count+=(e->tag==tag);
-   return count;
+      if(e->tag==tag)
+	 return true;
+   return false;
+}
+
+static bool IsBefore(unsigned id1,unsigned id2)
+{
+   // order with wrap-around
+   return id2 - id1 < id1 - id2;
+}
+
+bool SFtp::HasExpectBefore(unsigned id,Expect::expect_t tag)
+{
+   for(Expect *e=expect_queue.each_begin(); e; e=expect_queue.each_next())
+      if(e->tag==tag && IsBefore(e->request->GetID(),id))
+	 return true;
+   return false;
 }
 
 Glob *SFtp::MakeGlob(const char *pat)
@@ -1167,13 +1187,16 @@ int SFtp::Read(Buffer *buf,int size)
       return error_code;
    if(mode==CLOSED)
       return 0;
-   if(state==DONE && !(file_buf && file_buf->Size()>0))
+   if(state==DONE && (!file_buf || (file_buf->Size()==0 && file_buf->Eof())))
       return 0;	  // eof
    if(state==FILE_RECV)
    {
       // keep some packets in flight.
       int limit=(entity_size>=0?max_packets_in_flight:max_packets_in_flight_slow_start);
-      if(RespQueueSize()<limit && !file_buf->Eof())
+      int ooo_queue_available=max_out_of_order-ooo_chain.count();
+      int current_in_flight=RespQueueSize();
+      if(RespQueueSize()<limit && !file_buf->Eof()
+         && current_in_flight < ooo_queue_available)
       {
 	 // but don't request much after possible EOF.
 	 if(entity_size<0 || request_pos<entity_size || RespQueueSize()<2)
