@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2020 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <config.h>
 #include "SSH_Access.h"
 #include "misc.h"
+#include "ascii_ctype.h"
 #include <algorithm>
 #include <cctype>
 #include <string>
@@ -47,16 +48,17 @@ static bool begins_with(const char *b,const char *e,const char *suffix)
    return (e-b>=len && !strncasecmp(b,suffix,len));
 }
 
-static bool contains(char const * begin, char const * end, std::string const & needle)
+struct nocase_eq
 {
-   struct nocase_eq
+   inline bool operator() (char lhs, char rhs) const
    {
-      inline bool operator() (char lhs, char rhs) const
-      {
-         return std::tolower(lhs) == std::tolower(rhs);
-      };
+      return c_tolower(lhs) == c_tolower(rhs);
    };
-   return std::search(begin, end, needle.begin(), needle.end(), nocase_eq()) != end;
+};
+
+static bool contains(char const *begin, char const *end, char const *needle)
+{
+   return std::search(begin, end, needle, needle+strlen(needle), nocase_eq()) != end;
 }
 
 static bool IsPasswordPrompt(const char *b,const char *e)
