@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: mgetJob.h,v 1.14 2011/04/29 04:58:29 lav Exp $ */
-
 #ifndef MGETJOB_H
 #define MGETJOB_H
 
@@ -28,27 +26,21 @@
 
 class mgetJob : public GetJob
 {
-   Ref<GlobURL> rg;
-   JobRef<mkdirJob> mkdir_job;
-   Ref<ArgV> mkdir_args;
-   int mkdir_base_arg;
-   Ref<ArgV> m_args;
+   Ref<GlobURL> glob;
+   xqueue_m<char> wcd;
    xstring_c output_dir;
 
-   bool	 make_dirs;
-   void	 make_directory(const char *d);
-
-   void LocalGlob(const char *p);
+   FileAccessRef local_session;
 
 public:
    int	 Do();
    xstring& FormatStatus(xstring&,int,const char *);
    void	 ShowRunStatus(const SMTaskRef<StatusLine>&);
 
-   mgetJob(FileAccess *session,ArgV *args,bool c,bool md);
+   mgetJob(FileAccess *session,const ArgV *args,bool c,bool md);
    ~mgetJob();
 
-   void OutputDir(char *o) { output_dir.set_allocated(o); }
+   void OutputDir(const char *o) { output_dir.set(o); }
 };
 
 #endif // MGETJOB_H
