@@ -5,6 +5,7 @@
 #include <config.h>
 #include <unistd.h>
 #include "FileAccess.h"
+#include "log.h"
 
 char *program_name;
 
@@ -12,13 +13,20 @@ int main(int argc,char **argv)
 {
    program_name=argv[0];
 
-   FileAccess *f=FileAccess::New("ftp","ftp.redhat.com");
+   Log::global=new Log("debug");
+   ResMgr::Set("log:level",0,"5");
+   ResMgr::Set("log:enabled",0,"true");
+
+   ResMgr::Set("ftp:use-stat-for-list",0,"true");
+   ResMgr::Set("ftp:sync-mode",0,"false");
+
+   FileAccess *f=FileAccess::New("ftp","mirror.yandex.ru");
    if(!f)
    {
       fprintf(stderr,"ftp: unknown protocol, cannot create ftp session\n");
       return 1;
    }
-   f->Open("/redhat",f->LONG_LIST);
+   f->Open("/",f->LONG_LIST);
    Buffer buf;
    for(;;)
    {
